'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _dom = require('../../dom');

var _dom2 = _interopRequireDefault(_dom);

var _util = require('../util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ImageItem = function ImageItem(props) {
    (0, _classCallCheck3.default)(this, ImageItem);

    _initialiseProps.call(this);

    this.props = props;
    var _props = this.props,
        slide = _props.slide,
        data = _props.data;

    this.type = 'image';
    this.root = document.createElement("DIV");
    this.root.className = 'snk-slide-image-item';
    this.scale = 1;
    var item = data.item;
    var snkrotateValue = item.Vo.snkrotateValue;

    this.rotateValue = snkrotateValue || 0;
    var src = data.src;
    if (slide.props.getSrc) {
        src = slide.props.getSrc({
            data: data
        });
    }
    this.loadImage(src);
};

var _initialiseProps = function _initialiseProps() {
    var _this = this;

    this.rotate = function (value) {
        _this.rotateValue = _this.rotateValue + value;
        if (_this.rotateValue >= 360) {
            _this.rotateValue = 0;
        }
        if (_this.rotateValue === -90) {
            _this.rotateValue = 270;
        }
        _this.reset();
    };

    this.bindEvent = function () {
        var _props2 = _this.props,
            isSupportTouch = _props2.isSupportTouch,
            eventDict = _props2.eventDict;

        if (!isSupportTouch) {
            _dom2.default.bind({
                type: 'mousewheel',
                dom: _this.root,
                callback: function callback(e) {
                    _this.wheel(e);
                }
            });
            _dom2.default.bind({
                type: 'DOMMouseScroll',
                dom: _this.root,
                callback: function callback(e) {
                    _this.wheel(e);
                }
            });
            _dom2.default.bind({
                type: 'mousedown',
                dom: _this.img,
                callback: function callback(e) {
                    _this.onMouseDown(e);
                }
            });
            _dom2.default.bind({
                type: 'mousemove',
                dom: _this.img,
                callback: function callback(e) {
                    _this.onMouseMove(e);
                }
            });
            _dom2.default.bind({
                type: 'mouseup',
                dom: _this.img,
                callback: function callback(e) {
                    _this.onMouseUp(e);
                }
            });

            _dom2.default.bind({
                type: 'mouseleave',
                dom: _this.img,
                callback: function callback(e) {
                    _this.onMouseUp(e);
                }
            });
        }
    };

    this.onMouseUp = function (e) {
        var slide = _this.props.slide;

        if (!slide.touchDisabled) {
            return;
        }
        e.preventDefault();
        e.stopPropagation();
        _this.img.style.cursor = 'grab';
        _this.isMouseDown = false;
    };

    this.onMouseMove = function (e) {
        var slide = _this.props.slide;

        if (!slide.touchDisabled || !_this.isMouseDown) {
            return;
        }
        e.preventDefault();
        e.stopPropagation();
        var diff = {
            x: e.pageX - _this.start.x,
            y: e.pageY - _this.start.y
        };
        _this.img.style.left = _this.start.left + diff.x + 'px';
        _this.img.style.top = _this.start.top + diff.y + 'px';
    };

    this.onMouseDown = function (e) {
        var slide = _this.props.slide;

        if (!slide.touchDisabled) {
            return;
        }
        e.preventDefault();
        e.stopPropagation();
        _this.isMouseDown = true;
        _this.img.style.cursor = 'grabbing';
        _this.start = {
            x: e.pageX,
            y: e.pageY,
            top: parseInt(_this.img.style.top),
            left: parseInt(_this.img.style.left)
        };
    };

    this.setSlideTouchDisabled = function (isDisabled) {
        var slide = _this.props.slide;

        slide.touchDisabled = isDisabled;
        if (isDisabled) {
            _this.img.style.cursor = 'grab';
        } else {
            _this.img.style.cursor = 'default';
        }
    };

    this.wheel = function (e) {
        e.preventDefault && e.preventDefault();
        // wheelDeltaY > 0 向前
        var detail = 0;
        if (e.wheelDelta) {
            detail = e.wheelDelta;
        } else if (e.detail) {
            detail = 0 - e.detail;
        }

        if (detail > 0) {
            // 向上滚动
            _this.scaleMethod(_this.scale + 0.1, e);
        } else if (detail < 0) {
            // 向下滚动
            _this.scaleMethod(_this.scale - 0.1, e);
        }
    };

    this.scaleMethod = function (scale, e) {
        var slide = _this.props.slide;

        var BeforeScalePointX = e.offsetX;
        var BeforeScalePointY = e.offsetY;
        scale = scale < 0 ? 0.1 : scale;

        var InOrignImagePointX = BeforeScalePointX / _this.scale;
        var InOrignImagePointY = BeforeScalePointY / _this.scale;
        _this.scale = Math.round(scale * 10) / 10;
        var beforeLeft = parseInt(_this.img.style.left);
        var beforeTop = parseInt(_this.img.style.top);
        var AfterScalePointX = InOrignImagePointX * _this.scale;
        var AfterScalePointY = InOrignImagePointY * _this.scale;
        var curWidth = _this.imgOrginDomSize.width * _this.scale;
        var curHeight = _this.imgOrginDomSize.height * _this.scale;
        _this.img.style.width = curWidth + 'px';
        _this.img.style.height = curHeight + 'px';
        _this.img.style.left = beforeLeft - (AfterScalePointX - BeforeScalePointX) + 'px';
        _this.img.style.top = beforeTop - (AfterScalePointY - BeforeScalePointY) + 'px';

        if (_this.resetTimeout) {
            window.clearTimeout(_this.resetTimeout);
            _this.resetTimeout = null;
        }
        _this.resetTimeout = setTimeout(function () {
            if (_this.scale <= 1) {
                _this.setSlideTouchDisabled(false);
                _this.reset();
            } else {
                _this.setSlideTouchDisabled(true);

                var _img$getBoundingClien = _this.img.getBoundingClientRect(),
                    imgWidth = _img$getBoundingClien.width,
                    imgHeight = _img$getBoundingClien.height;

                var rootWidth = _this.root.offsetWidth;
                var rootHeight = _this.root.offsetHeight;
                if (imgWidth < rootWidth) {
                    _this.img.style.left = (rootWidth - imgWidth) / 2 + 'px';
                } else if (_this.img.offsetLeft > 0) {
                    _this.img.style.left = 0;
                } else if (imgWidth + _this.img.offsetLeft < rootWidth) {
                    _this.img.style.left = rootWidth - imgWidth + 'px';
                }
                if (imgHeight < rootHeight) {
                    _this.img.style.top = (rootHeight - imgHeight) / 2 + 'px';
                } else if (_this.img.offsetTop > 0) {
                    _this.img.style.top = 0;
                } else if (imgHeight + _this.img.offsetTop < rootHeight) {
                    _this.img.style.top = rootHeight - imgHeight + 'px';
                }
            }
        }, 150);
    };

    this.onResize = function (size) {
        var slide = _this.props.slide;

        _this.setSlideTouchDisabled(false);
        _this.scale = 1;
        _this.autoResizeImage(size.width, size.height, _this.imgOriginWidth, _this.imgOriginHeight, _this.img);
    };

    this.reset = function () {
        var slide = _this.props.slide;

        _this.setSlideTouchDisabled(false);
        _this.scale = 1;
        _this.img.style.left = 0;
        _this.img.style.top = 0;
        _this.autoResizeImage(_this.root.offsetWidth, _this.root.offsetHeight, _this.imgOriginWidth, _this.imgOriginHeight, _this.img);
    };

    this.autoResizeImage = function (maxWidth, maxHeight, w, h, objImg) {
        var slide = _this.props.slide;

        _this.setSlideTouchDisabled(false);
        _this.scale = 1;
        _this.img.className = 'snk-slide-image';
        _this.imgOrginDomSize = _util2.default.autoResizeImage(maxWidth, maxHeight, w, h, objImg, _this.rotateValue, _this.root);
    };

    this.loadImage = function (src) {
        _this.root.innerHTML = '';
        var loadingSpan = document.createElement("SPAN");
        loadingSpan.innerHTML = '加载中...';
        loadingSpan.className = 'snk-slide-image-loading';
        _this.root.appendChild(loadingSpan);
        _this.img = new Image();
        _this.isUserWidth = false;
        _this.img.className = 'snk-slide-image';
        _this.img.onload = function () {
            _this.scale = 1;
            setTimeout(function () {
                _this.imgOriginWidth = _this.img.width;
                _this.imgOriginHeight = _this.img.height;
                _this.wrapperWidth = _this.root.offsetWidth;
                _this.wrapperHeight = _this.root.offsetHeight;
                _this.root.innerHTML = '';
                _this.root.appendChild(_this.img);
                _this.autoResizeImage(_this.wrapperWidth, _this.wrapperHeight, _this.imgOriginWidth, _this.imgOriginHeight, _this.img);
                _this.bindEvent(_this.img);
            }, 10);
        };
        _this.img.onerror = function () {
            _this.root.innerHTML = '';
            var reload = document.createElement('SPAN');
            reload.className = 'snk-slide-image-loading';
            reload.innerHTML = '<span>网络异常,</span><span style="color:#2dd3d4;cursor:pointer;">点击重新加载</span>';
            _this.root.appendChild(reload);
            _dom2.default.bind({
                dom: reload,
                type: 'click',
                callback: function callback() {
                    _this.loadImage(src);
                }
            });
        };
        _this.img.src = src;
    };
};

exports.default = ImageItem;