'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _dom = require('../../dom');

var _dom2 = _interopRequireDefault(_dom);

var _treeSection = require('./treeSection');

var _treeSection2 = _interopRequireDefault(_treeSection);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TreeItem = function TreeItem(props) {
  var _this = this;

  (0, _classCallCheck3.default)(this, TreeItem);

  _initialiseProps.call(this);

  this.props = props;
  var _props = this.props,
      data = _props.data,
      tree = _props.tree,
      popover = _props.popover,
      preDataArr = _props.preDataArr,
      index = _props.index,
      parentItem = _props.parentItem;

  this.parentItem = parentItem;
  this.isSelected = false;
  this.id = tree.getUniqueSeed();
  tree.nodeDict[this.id] = this;
  this.isOpen = index === 0;
  this.root = document.createElement('LI');
  this.curPreArr = [].concat((0, _toConsumableArray3.default)(preDataArr || []));
  this.curPreArr.push(data);
  data.nodeInstance = this;
  this.createItemLayout(data);
  this.createChildSection(data);
  var params = {
    data: data,
    tree: tree,
    node: this,
    preDataArr: this.curPreArr,
    popover: popover
  };
  _dom2.default.bind({
    type: 'click',
    dom: this.root,
    prevent: true,
    stop: true,
    callback: function callback() {
      if (tree.props.itemClickToToggle === true) {
        _this.toggle();
      }
      if (tree.props.onBeforeItemSelect) {
        if (tree.props.onBeforeItemSelect(params) !== false) {
          _this.select();
        }
      } else {
        _this.select();
      }
      if (tree.props.onItemClick) {
        tree.props.onItemClick(params);
      }
    }
  });
  if (tree.props.onItemCreate) {
    tree.props.onItemCreate(params);
  }
};

var _initialiseProps = function _initialiseProps() {
  var _this2 = this;

  this.createChildSection = function (data) {
    var children = data.children;
    var _props2 = _this2.props,
        tree = _props2.tree,
        index = _props2.index,
        popover = _props2.popover;

    if (!children) {
      return;
    }
    _this2.section = new _treeSection2.default({
      tree: tree,
      data: children,
      parentSection: _this2.parentSection,
      parentItem: _this2,
      index: index + 1,
      preDataArr: [].concat((0, _toConsumableArray3.default)(_this2.curPreArr)),
      popover: popover
    });
    _this2.root.appendChild(_this2.section.root);
  };

  this.select = function () {
    var tree = _this2.props.tree;

    if (tree.selectedNode) {
      tree.selectedNode.unSelect();
    }
    tree.selectedNode = _this2;
    _this2.isSelected = true;
    _this2.setClassName();
  };

  this.toSelect = function () {
    var _props3 = _this2.props,
        data = _props3.data,
        tree = _props3.tree,
        popover = _props3.popover;

    var params = {
      data: data,
      tree: tree,
      node: _this2,
      preDataArr: _this2.curPreArr,
      popover: popover
    };
    _this2.select();
    if (tree.props.onItemClick) {
      tree.props.onItemClick(params);
    }
  };

  this.unSelect = function () {
    _this2.isSelected = false;
    _this2.setClassName();
  };

  this.setClassName = function () {
    var _props4 = _this2.props,
        data = _props4.data,
        tree = _props4.tree;

    var hasChildren = data.children && data.children.length > 0 && tree.props.setionItemNoHover;
    _this2.itemWrapper.className = 'snk-tree-item' + (hasChildren ? '-header' : '') + ' ' + (_this2.isSelected ? 'snk-tree-item-selected' : '') + ' snk-tree-item-' + (_this2.isOpen ? 'open' : 'close');
  };

  this.createItemLayout = function (data) {
    var _ref = data || { label: '' },
        label = _ref.label,
        children = _ref.children;

    var _props5 = _this2.props,
        index = _props5.index,
        tree = _props5.tree;
    var _tree$props = tree.props,
        isReact = _tree$props.isReact,
        onBeforeLabelCreate = _tree$props.onBeforeLabelCreate;

    _this2.itemWrapper = document.createElement('DIV');
    _this2.itemWrapper.setAttribute('node-id', _this2.id);
    _this2.setClassName();
    _this2.itemWrapper.style.paddingLeft = index * tree.offset + 'px';
    if (children && children.length > 0) {
      var toggleIcon = document.createElement('SPAN');
      toggleIcon.className = 'snk-tree-toggle';
      _this2.itemWrapper.appendChild(toggleIcon);
      if (tree.props.itemClickToToggle !== true) {
        _dom2.default.bind({
          dom: toggleIcon,
          type: 'click',
          stop: true,
          prevent: true,
          callback: function callback() {
            _this2.toggle();
          }
        });
      }
    } else {
      var toggleIconPlaceHolder = document.createElement('SPAN');
      toggleIconPlaceHolder.className = 'snk-tree-toggle-empty';
      _this2.itemWrapper.appendChild(toggleIconPlaceHolder);
    }
    var params = {
      tree: tree,
      itemDom: _this2.itemWrapper,
      data: data,
      itemInstance: _this2
    };
    if (isReact && onBeforeLabelCreate) {
      var reactBeforeLabelDom = document.createElement('span');
      reactBeforeLabelDom.className = 'snk-tree-span-beforelabel';
      _this2.itemWrapper.appendChild(reactBeforeLabelDom);
      params.reactBeforeLabelDom = reactBeforeLabelDom;
    }
    tree.beforeLabelCreate(params);

    _this2.countLabel = document.createElement('span');
    _this2.countLabel.className = 'snk-tree-span-countlabel';
    _this2.countLabel.innerHTML = 0;
    _this2.itemWrapper.appendChild(_this2.countLabel);

    var labelSpan = document.createElement('SPAN');

    labelSpan.className = 'snk-tree-label';
    labelSpan.innerHTML = label;
    labelSpan.setAttribute('title', label);
    _this2.itemWrapper.appendChild(labelSpan);

    _this2.root.appendChild(_this2.itemWrapper);
  };

  this.toggle = function () {
    if (_this2.isOpen) {
      _this2.close();
    } else {
      _this2.open();
    }
  };

  this.open = function () {
    if (!_this2.section) {
      return;
    }
    _this2.isOpen = true;
    _this2.setClassName();
    _this2.section.open();
  };

  this.close = function () {
    if (!_this2.section) {
      return;
    }
    _this2.isOpen = false;
    _this2.setClassName();
    _this2.section.close();
  };
};

exports.default = TreeItem;