
import Services from '../components/services/HttpService';

class AjaxPrinter {
  constructor(props) {
    this.props = props;
    const { initParams } = this.props;
    this.root = $("<div class='snk-bh-camera-root'/>");
    this.initParams = initParams || [];
    this.upLoadData = {
      printName: null, // 打印机目标
      printSize: null, // 打印尺寸 如：A4
      printQuality: null, // 打印质量
      printLayout: null, // 打印布局 如：横向
      printTwoSide: null, // 打印是否双面，默认否
      printNum: 1, // 打印份数，默认为1份
    };
    this.isPrintLoging = false;
  }

  initLayout = () => {
    const { dialog, ok, title } = this.props;
    console.log(!this.layout)
    if(!this.layout) {
      // this.layout = new CommonLayout({ dialog, title: title || "拍照", parent:this, ok });
      this.topArea = $("<div class='snk-bh-croot-top'></div>");
      this.createTopArea({ dialog, title: title || "打印" })

      this.printBtn = $("<span class='snk-print-botton'>打印当前文件</span>");
      this.printBtn.bind('click', () => {
        this.doPrint('one');
      });
      this.printAllBtn = $("<span class='snk-print-botton'>批量打印文件</span>");
      this.printAllBtn.bind('click', () => {
        this.doPrint('all');
      });
      const midArea = $("<div class='snk-print-body'></div>");
      const selectData = [
        { type: 'printName', name: '选择目标打印机' },
        // { type: 'printSize', name: '纸张尺寸' },
        // { type: 'printQuality', name: '打印质量' },
        // { type: 'printLayout', name: '打印布局' },
        // { type: 'printTwoSide', name: '是否双面打印' },
      ]
      for (let index = 0; index < selectData.length; index++) {
        const element = selectData[index];
        this.createPrintList(midArea, element);
      }
      // this.createInput(midArea, { name: '打印份数', type: 'printNum' });
      midArea.append(this.printBtn).append(this.printAllBtn);
      this.root.append(this.topArea).append(midArea);
    }
  }

  createTopArea = ({ title, dialog }) => {
    const { actionInstance } = dialog;
    const closeBtn = $("<span class='snk-bh-header-close'>关闭</span>");
    closeBtn.bind('click', () => {
      if(actionInstance.cancelCheck()) {
        dialog.close();
      }
    });
    this.title =  $(`<span class='snk-bh-header-title'>${title}</span>`);
    this.topArea.append(this.title).append(closeBtn);
  }

  createPrintList = (root, element) => {
    const _that = this;
    this[element.type] = $('<select />');
    this[element.type].bind('change',(e) => {
      const valIndex = e.target.value;
      _that._fillResolutionSelect(valIndex, element.type);
      _that.saveLocalData(element.type, valIndex);
    });
    const selectArea = $("<div class='snk-print-select-body'></div>");
    selectArea.append(`<div class='snk-print-select-laber'>${element.name}</div>`).append(this[element.type]);
    root.append(selectArea);
  }

  createInput = (root, element) => {
    const _that = this;
    _that[element.type] = $('<input class="snk-print-input" />');
    _that[element.type].bind('keydown', (e) => {
      if(e.keyCode === 13) {
        e.preventDefault();
        e.stopPropagation();
        _that[element.type].blur();
      }
    });
    _that[element.type].bind('blur', () => {
      const v = $.trim(_that[element.type].val());
      console.log(v)
      if (!v || !/^([1-9][0-9]{0,1}|0)$/.test(v)) {
        _that[element.type].val(1)
        _that._fillResolutionSelect(1, element.type);
      } else {
        _that[element.type].val(parseInt(v))
        _that._fillResolutionSelect(parseInt(v), element.type);
      }
    });
    const inputArea = $("<div class='snk-print-select-body'></div>");
    inputArea.append(`<div class='snk-print-select-laber'>${element.name}</div>`).append(_that[element.type]);
    root.append(inputArea);
    _that[element.type].val(parseInt(1))
  }


  cancelCheck = () => {
    // const re = this.getResult();
    // if(re.length ===0) {
    //   return true;
    // }
    // return window.confirm("存在操作记录，是否继续退出?");
    return true;
  }

  delUploadData = (type) => { // 处理上传参数
    const _that = this;
    const { urlData, index } = _that.initParams;
    if (!urlData) {
      return;
    }
    const initParamsLen = urlData.length;
    const newData = [];
    console.log(_that.upLoadData)
    let newObj = { filePaths: [], printerName: _that.upLoadData.printName, copies: '1' };
    if (type && type === 'one') { // 打印当前文件
      newObj.filePaths = [urlData[index || 0]];
      return newObj;
    }

     // 批量打印文件
    for (let index1 = 0; index1 < initParamsLen; index1 += 1) {
      newObj.filePaths = [urlData[index1]];
      newData.push(newObj);
    }
    return newData;
  }

  doPrint = (type) => {
    const _that = this;
    const { dialog } = this.props;
    if (this.isPrintLoging) {
      return;
    }
    // const printName = _that.upLoadData.printName;
    // if (printName.includes('导出') || printName.includes('to PDF') || printName.includes('Foxit')) {
    //   alert(`不能使用-${printName}进行打印`);
    //   return;
    // }
    _that.showLoading()
    const upLoadData = _that.delUploadData(type);
    if ((type === 'all' && (!upLoadData || upLoadData.length === 0))
      || (type === 'one' && (!upLoadData.filePaths || upLoadData.filePaths.length === 0))) {
      alert('找不到打印文件');
      _that.hideLoading();
      return;
    }
    try {
      Services.evenloop(
        upLoadData, Services.doPrint,
        0,
        () => {
          _that.hideLoading();
          alert('推送至打印机成功')
          dialog.close();
        },
        (errorData, index) => {
          _that.hideLoading();
          console.log(errorData)
          if (errorData && errorData.code === 500 && errorData.message) {
            alert(errorData.message)
            dialog.close();
            // alert(`第${index + 1}个文件打印失败`)
          }
       });
    } catch (error) {
      console.log(error);
    }
    console.log(upLoadData);
  }

  logingFn = (isLoging) => {
    if (isLoging) {
      this.isPrintLoging = isLoging;
      this.printBtn.addClass('disabled');
      this.printAllBtn.addClass('disabled');
    } else {
      this.isPrintLoging = isLoging;
      this.printBtn.removeClass('disabled');
      this.printAllBtn.removeClass('disabled');
    }
  }

  saveLocalData = (item, value) => { // 持久化到浏览器中
    if (!item || !value) {
      console.error('请填写完整持久化信息');
    }
    const storageData = JSON.stringify(value);
    localStorage.setItem(item, storageData);
  }

  getLocalData = (item) => { // 获取浏览器中的持久化
    if (!item) {
      console.error('请填写完整持久化信息');
    }
    const storageData = localStorage.getItem(item);
    return JSON.parse(storageData);
  }

  fillSelects = (data, component, type) => { // 获得设备列表后回填
    const { dialog } = this.props;
    const { message } = dialog;
    message.hide();
    dialog.wrapper.addClass('snk-bh-wrapper-print');
    component.empty();
    this.cameraListDataSource = data;
    let index = 0;
    for(let i = 0, j = data.length; i < j; i+=1) {
      const MonikerString = data[i];
      if (MonikerString.includes('EPSON') || MonikerString.includes('Canon')) {
        index = i;
      }
      component.append(`<option value='${MonikerString}'>${MonikerString}</option>`);  
    }
    const oldData = this.getLocalData(type);
    component.val(oldData || data[index])
    this._fillResolutionSelect(oldData || data[index], type);
  }

  _fillResolutionSelect = (value, type) => { // 保存选择的数据
    this.upLoadData[type] = value;
  }

  onReciveMessage = (jsonData) => {
    const { dialog } = this.props;
    if(jsonData){
      if(jsonData.type === 'printName') {
        dialog.callDllLogicSuccess("callcamerasuccess");
        this.reciveCamerList({ jsonData });
      } else if(jsonData.type === 'Camera_Connect') {
        
      }else if(jsonData.type === 'Camera_GetImage') {
          
      }else if(jsonData.type === 'Get_Version') {
          console.log(jsonData.data);
      } else if(jsonData.type === 'Image_Ruihua') {
          // console.log(jsonData.imgId);
          this.layout.midLeftArea.append($(`<img src='${jsonData.data}'/>`));
      }
    } 
  }

  getUpdateArea = () => {
    return this.layout.bottomArea;
  }

  setErrorMessage = (params) => {
    const { dialog } = this.props;
    const { message } = dialog;
    message.show(params);
  }

  showLoading = (message1) => {
    const { dialog } = this.props;
    const { message } = dialog;
    message.show({
      isCloseDialog: true,
      html: message1 || '正在打印中...',
    });
  }

  hideLoading = () => {
    const { dialog } = this.props;
    const { message } = dialog;
    message.hide();
  }

  showError = () => {
    this.loadingWrapper.css({"display":"flex"});
    this.loadingWrapper.empty();
    this.loadingWrapper.append($(`<span>打印机可能被其他（程序应用，网页）占用,请关闭相关应用然后重试，或者切换其他摄像头！</span>`));
  }

  reciveCamerList = ({ jsonData }) => {
    const { code, data, message, type } = jsonData;
    if(code === 200) {
      if(data.length > 0) {
        this.fillSelects(data, this.printName, type);
        // this.fillSelects(['是', '否'], this.printTwoSide, 'printTwoSide');
        // this.fillSelects(['纵向', '横向'], this.printLayout, 'printLayout');
      } else {
        this.setErrorMessage({
          isCloseDialog: true,
          html: "没有发现打印机相关设备,请准备好相关硬件然后重试！"
        });
      }
    } else {
      this.setErrorMessage({
        isCloseDialog: true,
        html: message || "获取打印机信息失败！"
      });
    }
  }


  disConnect = () => {
    // if(this.ws) {
    //   try{
    //    this.ws.send(JSON.stringify({ action:'Camera_Close' }));
    //   }catch(e){}
    // }
    // this.clearGetImageInterval();
  }

  connectSuccess = ({ ws }) => {
    this.initLayout();
    this.onReciveMessage({ type: 'printName',  code: 200, data: ws })
  }

  hasClass = (obj, cls) => {
    return obj.className.match(new RegExp(`(\\s|^)${cls}(\\s|$)`));
  }
}

export default AjaxPrinter;