import PreViewDialog from './cameraScannerImgList/previewDialog';

class CommonLayout {
  constructor(props) {
    this.props = props;
    this.initLayout();
  }
  initLayout = () => {
    this.root = $("<div class='snk-bh-croot'></div>");
    this.topArea  = $("<div class='snk-bh-croot-top'></div>");
    this.createTopArea();
    const midArea = $("<div class='snk-bh-croot-mid'></div>");
    this.midAreaInner = $("<div class='snk-bh-croot-mid-inner'></div>");
    this.bottomArea = $("<div class='snk-bh-croot-bottom'></div>");
    this.createBottomArea();
    const midLeftArea = $("<div class='snk-bh-croot-mid-left'></div>");
    this.createLeftArea(midLeftArea);
    this.midMidArea = $("<div class='snk-bh-croot-mid-mid'></div>");
    this.midRightArea =  $("<div class='snk-bh-croot-mid-right'></div>");
    midArea.append(this.midAreaInner);
    this.midAreaInner.append(midLeftArea).append(this.midMidArea).append(this.midRightArea);
    this.root.append(this.topArea).append(midArea).append(this.bottomArea);
  }

  showPreview = (item) => {
    const { dialog } = this.props;
    const { actionInstance } = dialog;
    if(!this.previewDialogInstance) {
      this.previewDialogInstance = new PreViewDialog();
      this.root.append(this.previewDialogInstance.root);
      this.root.append(this.previewDialogInstance.header);
    }
    this.previewDialogInstance.show(item);
    if(actionInstance.onShowPreView) {
      actionInstance.onShowPreView();
    }
  }

  hidePreview = () => {
    if(this.previewDialogInstance) {
      this.previewDialogInstance.hide();
    }
  }

  createLeftArea = (wrapper) => {
    const leftTitleWrapper = $("<div class='snk-bh-croot-mid-left-top'></div>");
    this.secondTitle = $("<span></span>");
    leftTitleWrapper.append(this.secondTitle);
    this.midLeftArea =  $("<div class='snk-bh-croot-mid-left-bottom'></div>");
    wrapper.append(leftTitleWrapper).append(this.midLeftArea);
  }

  setSecondTitle = (str) => {
    this.secondTitle.empty();
    this.secondTitle.append(str);
  }

  createTopArea = () => {
    const { title, dialog } = this.props;
    const { actionInstance } = dialog;
    const closeBtn = $("<span class='snk-bh-header-close'>关闭</span>");
    closeBtn.bind('click', () => {
      if(actionInstance.cancelCheck()) {
        dialog.close();
      }
    });
    this.title =  $(`<span class='snk-bh-header-title'>${title}</span>`);
    this.topArea.append(this.title).append(closeBtn);
  }


  createBottomArea = () => {
    const { dialog,parent, ok } = this.props;
    const { actionInstance } = dialog;
    const btnCancel = $("<button class='snk-bh-croot-cancelbtn' >取消</button>");
    btnCancel.bind('click',()=>{
      if(actionInstance.cancelCheck()) {
        dialog.close();
      }
    });
    this.btnOk = $("<button class='snk-bh-croot-okbtn' disabled='disabled'>确定</button>");
    this.btnOk.bind("click", () => {
      const re = actionInstance.okClick();
      if(re !== false) {
        ok({ result: re });
        dialog.close();
      }
    });
    
    const openHistoryBtn = $("<span title='打开查看本地电脑历史操作图片保存位置（只保留最近五天的操作）' class='snk-bwh-openhistory'>查看本地记录</span>");
    openHistoryBtn.bind('click', () => {
      if(parent.ws) {
        parent.ws.send(JSON.stringify({ action:'Open_Folder' }));
      }
    });
    this.bottomArea.append(openHistoryBtn).append(btnCancel).append(this.btnOk);
  }

  setOkBtnState = (resultLength) => {
    if(resultLength === 0) {
      this.btnOk.attr("disabled", "disabled");
    } else {
      this.btnOk.removeAttr("disabled");
    }
  }
}

export default CommonLayout;