import AjaxPrinter from './ajax-printer';
import Message from './message';
import protocolcheck from './protocolcheck';
import Services from '../components/services/HttpService';

class Dialog {
  constructor(props) {
    this.props = props;
    this.connectTimeoutID = null;
    this.succeeded = false;
    this.actionInstance = null;
    this.tryConnectCount = 0;
    this.hasSuccessCount = 0;
    this.websocketInstance = null;
    this.hasError = false;
    this.step = 'init';
    this.initLayout();
    this.timeoutArr = [1,1,500, 750, 800, 800];
  }

  callDllLogicSuccess = (step) => {
    this.step = step;
  }

  initLayout = () => {
    const { action, ok, env, token, initParams, reStartCallBack, title, hideLianPai, hostUrl } = this.props;
    this.message = new Message({ dialog: this, env, reStartCallBack, downLoadHost: hostUrl });
    this.bk = $(`<div class='snk-bh-bk snk-bh-bk-${action}'></div>`);
    this.wrapper = $(`<div class='snk-bh-wrapper snk-bh-wrapper-${action}'></div>`);
    if (action ==='ajax-print') { // ajax请求插件
      this.actionInstance = new AjaxPrinter({ dialog: this, ok, env, initParams, token });
    }
    this.wrapper.append(this.message.root).append(this.actionInstance.root);
  }

  getResult = () => {
    return this.actionInstance.getResult();
  }

  clearConnectTimeout = () => {
    if(this.connectTimeoutID) {
      window.clearTimeout(this.connectTimeoutID);
      this.connectTimeoutID = null;
    }
  }
  connect = () => {
    const { port, env ,action } = this.props;
    this.clearConnectTimeout();
    this.checkUpdateSuccess = false;
    this.limitMax = 20;
    this.message.showLoading(`正在检测本地硬件环境(${Math.floor(this.tryConnectCount/2)}/${this.limitMax/2})。<br />
    如果还没有安装打印助手,请点击根据电脑系统下载安装:</span><br />
    (<a style='font-weight:bold;' download href='${this.message.dowloadURL}'>Win 64位</a>)、(<a style='font-weight:bold;' download href='${this.message.dowloadURL1}'>Win 32位</a>)
    <br /><span style='color:red'>安装运行后再重试</span>`);
    this.tryConnectCount += 1;
    this.connectTimeoutID = window.setTimeout(() => {
      if(this.tryConnectCount > this.limitMax) {
        this.showConnectError();
        return;
      }
      if (action ==='ajax-print') {
        const _that = this;
        // this.message.hide();
        // _that.connectSuccess([1, 2, 3]);
        Services.getPrintList().then((data)=> {
          _that.connectSuccess(data);
          _that.clearConnectTimeout();
        }).catch(()=> {
          _that.disConnect();
        });
      }
      if (!this.websocketInstance) {
        return;
      }
    }, this.timeoutArr[this.tryConnectCount]||1000);
  }
  
  disposeConnect = () => {
    this.clearConnectTimeout();
    this.actionInstance.disConnect();
    if(this.websocketInstance) {
      this.websocketInstance.onclose = () => {}
      this.websocketInstance.close();
      this.websocketInstance = null;
    }
  }

  disConnect = () => { 
    if(this.tryConnectCount === 1) {
      this.callElectron();
    }
    window.onbeforeunload = () => {}
    this.clearConnectTimeout();
    if(this.succeeded) {
      this.succeeded = false;
      this.message.showDisConnect();
    }else {
      this.connect();
    }
    this.actionInstance.ws = null;
    this.actionInstance.disConnect();
  }

  callElectron = () => {
    const { port, env, action, callElectronCallBack } = this.props;
    const app = action === 'ajax-print' ? 'haprinthandler' : 'snkwebhelper';
    const str = `${app}://?port=${port}&action=${action}&env=${env}}`;
    if(callElectronCallBack) {
      // 因为从iframe唤起的话会报跨域 那么就让程序决定 让上一级去唤起
      callElectronCallBack(str);
    } else {
      protocolcheck(str, // 检测有无安装
        () => {
      });
    }
  }

  onReciveMessage = (e,ws) => {
    var data = e.data;
    var jsonData = null;
    try{
      jsonData = JSON.parse(data);
      const { action, code, data:reData, message:reMessage } = jsonData;
      console.log(action+"  >>>>>>>>>>>")
      if(action === 'App_Update') {
    
      } else if(action ==='GLOBAL') {
        this.hasError = true;
        if(reMessage && reMessage.indexOf('The specified module could not be found')>=0) {
          // 缺少vc++ 2015
          this.message.showDownloadVC();
        } else {
          this.message.show({
            isCloseDialog: true,
            html: reMessage || '~',
          });
        }
      } else if(action === "App_Check_Update") {
        if(reData === true) {
         this.showHasUpdate({ws, message: this.message, topArea: this.actionInstance.getUpdateArea()});
        }
      }

    }catch(e) {
      jsonData = null;
    }
  
    if(!this.hasError) {
      this.actionInstance.onReciveMessage(e, jsonData);
    }
  }

  showHasUpdate = ({ws , message, topArea}) => {
    if(this.hasCreateUpdateBtn === true) {
      return;
    }
    const slable = $("<span class='blink snk-wbh-update-label'>检测到新版本</span>");
    const updateBtn = $("<span class='snk-wbh-update-btn'>点击更新</span>");
    updateBtn.bind("click",()=>{
      if(window.confirm("确定更新吗?")) {
        message.showLoading("正在更新....");
        updateBtn.hide();
        slable.hide();
        ws.send(JSON.stringify({ action:'App_Update', data: { } }));
      }
    });
    topArea.append(slable).append(updateBtn);
    this.hasCreateUpdateBtn = true;
  }

  connectSuccess = (ws) => {
    this.clearConnectTimeout();
    this.succeeded = true;
    this.tryConnectCount = 0;
    this.hasSuccessCount += 1;
    this.actionInstance.connectSuccess({ws});
    window.onbeforeunload = () => {
      this.disposeConnect();
    }
  }

  showConnectError = () => {
    this.clearConnectTimeout();
    this.message.showCallElectronFail();
  }

  show = () => {
    this.tryConnectCount = 0;
    $(document.body).append(this.bk).append(this.wrapper);
    this.connect();
  }

  reStart = () => {
    this.tryConnectCount = 0;
    this.connect();
  }

  close = () => {
    const { close } = this.props;
    this.disposeConnect();
    this.bk.remove();
    this.wrapper.remove();
    if(close){
      close();
    }
  }
}

export default Dialog;