import SNKDom from '../dom';
import './index.less';
import Item from './item';
import util from './util';
import TheumbBar from './thumbBar';

const transformInfo = util.getTransitionKeys();
const isSupportTouch = "ontouchend" in document ? true : false;
const eventDict = {
    start: isSupportTouch?'touchstart':'mousedown',
    move:isSupportTouch?'touchmove':'mousemove',
    end:isSupportTouch?'touchend':'mouseup',
    cancel:isSupportTouch?'touchcancel':'mouseleave'
};
class Slide {
    constructor(props){
        this.props = props || {};
        const { zIndex, isLoop} = this.props;
        this.offset = 0;
        this.isShow = false;
        this.isLoop = !!isLoop;
        this.currentIndex = 0;
        this.isInTransition = false;
        this.root = document.createElement("DIV");
        this.root.className = 'snk-slide-root';
        this.root.style.display = 'none';
        this.root.style.zIndex = zIndex || 1000;
        this.bk = document.createElement('DIV');
        this.bk.className = 'snk-slide-bk';
        this.bk.style.zIndex = zIndex - 11 || 800;
        this.bk.style.display = 'none';
        this.touchLimit = 60;
        this.touchDisabled = false;
        const innerWrapper = document.createElement("DIV");
        innerWrapper.className = 'snk-slide-inner';
        this.itemArrary = [];
        this._createActionWrapper(innerWrapper);
        this._createItemsWrapper(innerWrapper);
        this._createThumbBar(innerWrapper);
        this.root.appendChild(innerWrapper);
        document.body.appendChild(this.bk);
        document.body.appendChild(this.root);
        window.onresize = util.throttle(()=>{
            if(!this.isShow){return;}
            this.wrapperWidth = this.root.offsetWidth;
            this.wrapperHeight = this.root.offsetHeight;
            this.setPos();
            for(let i = 0,j=this.itemArrary.length;i<j;i+=1) {
                this.itemArrary[i].onResize({
                    width:this.wrapperWidth,
                    height: this.wrapperHeight,
                });
            }
        },400);
        this._bindEvent();
        this._createCloseBtn();
    }

    _createActionWrapper = (innerWrapper) => {
      const { renderTopActionArea } = this.props;
      if(!renderTopActionArea) {
        return;
      }
      this.topActionArea = document.createElement("DIV");
      this.topActionArea.className = 'snk-slide-action-wrapper';
      innerWrapper.appendChild(this.topActionArea);
      renderTopActionArea({
        wrapper: this.topActionArea,
        instance: this
      });
    }


    _createCloseBtn = () => {
        // const closeBtn = document.createElement("DIV");
        // closeBtn.className = 'snk-slide-close';
        // SNKDom.bind({
        //     type: 'click',
        //     dom: closeBtn,
        //     callback: () => {
        //         this.hide();
        //     },
        // });
        // this.topActionArea.appendChild(closeBtn);
    }

    _createItemsWrapper = (innerWrapper)=> {
        this.itemsWrapper = document.createElement("DIV");
        this.itemsWrapper.className = 'snk-slide-item-wrapper';
        const itemsWrapperInner = document.createElement("DIV");
        itemsWrapperInner.className = 'snk-slide-item-wrapper-inner';
        for(let i = 0;i<3;i+=1) {
            const item = new Item({
                wrapper:itemsWrapperInner,
                isSupportTouch,
                slide:this,
                eventDict,
            });
            const itemStyle = {};
            itemStyle['left'] = `${(100*(i-1))}%`;
            item.setStyle(itemStyle);
            this.itemArrary[i] = item;
        }
        this.itemsWrapper.appendChild(itemsWrapperInner);
        innerWrapper.appendChild(this.itemsWrapper);
    }

    _createThumbBar = (innerWrapper)=>{
        if(!this.props.thumbBarPosition) {
            return;
        }
        this.thumbBarInstance = new TheumbBar({
            wrapper:innerWrapper,
            slide:this,
            itemsWrapper:this.itemsWrapper,
            postion:this.props.thumbBarPosition,
        });
    }

    changeByDataItem = (data) => {
        for(let i = 0,j=this.data.length;i<j;i+=1) {
            if(this.data[i] === data) {
                this.setSelectedByIndex(i);
                break;
            }
        }
    }

    getCurrentData = () => {
      return this.data[this.currentIndex];
    }

    getCurrentItem = () => {
      console.log(">>>>>>>>>>>>>>>");
      console.log(this.sourceArr);
      console.log(this.currentIndex);
      console.log(this.itemArrary)
      return this.itemArrary[1];
    }

    currentIsLast = () => {
      if(this.data.length === 0 ){
        return true;
      }
      if(this.currentIndex>=this.data.length -1) {
        return true;
      }
      return false;
    }

    currentIsFirst = () => {
      if(this.data.length === 0 ){
        return true;
      }
      if(this.currentIndex === 0) {
        return true;
      }
      return false;
    }

    setSelectedByIndex = (_index) => {
        this.currentIndex = _index;
        if(isNaN(this.currentIndex)) {
            this.currentIndex = 0;
        }
        this.currentIndex = parseInt(this.currentIndex);
        if(this.currentIndex>=this.data.length) {
            this.currentIndex = 0;
        }
        this.sourceArr = [-1,this.currentIndex-1,-1];
        this.getNextSourceArr();
        this.bindDataSource();
        this.onChange({
            isInit:true,
            index:this.currentIndex,
            slide:this,
        });
    }

    _getTouchPos = (e) => {
        if(isSupportTouch) {
            const touches = e.touches[0];
            return {
                x: touches.pageX,
                y: touches.pageY
            };
        } else {
            return {
                x: e.pageX || e.screenX,
                y: e.pageY || e.screenY,
            };
        }
    }
    _bindEvent = () => {
        let startPointer = {},startTime = 0,startOffset = 0,isTouch = false,diffX;
        SNKDom.bind({
            type: eventDict.start,
            dom: this.itemsWrapper,
            callback: (e) => {
                if(this.touchDisabled) {
                    return;
                }

                // if(e.offsetX > this.wrapperWidth - 22) {
                //     // ie 滚动条
                //     return;
                // }
                // if(e.offsetY > this.wrapperHeight - 22) {
                //     return;
                // }

                e.preventDefault && e.preventDefault();
                if(this.isInTransition) {
                    return;
                }
                startPointer = this._getTouchPos(e);
                startTime = new Date().valueOf();
                startOffset = this.offset;
                isTouch = true;
                diffX = 0;
            }
        });
        SNKDom.bind({
            dom: this.itemsWrapper,
            type: eventDict.move,
            prevent:true,
            callback: (e)=>{
                if(this.touchDisabled || !this.sourceArr) {
                    return;
                }
                
                if(this.isInTransition) {
                    return;
                }
                if(!isTouch){
                    return;
                }
                
                let curPointer = this._getTouchPos(e);
                diffX = (curPointer.x - startPointer.x) + startOffset;
                if(diffX>0) {
                    if(this.sourceArr[0] === -1){
                        diffX = 0;
                        return;
                    }
                }else {
                    if(this.sourceArr[2] === -1){
                        diffX = 0;
                        return;
                    }
                }
                this.setPos(diffX);
            }
        });
        SNKDom.bind({
            dom: this.itemsWrapper,
            type: eventDict.end,
            callback: (e) => {
                if(this.isInTransition) {
                    return;
                }
                touchEnd();
                diffX = 0;
            }
        });
        const touchEnd = () => {
            if(this.touchDisabled) {
                return;
            }
            if(diffX > 0) {
                // 右
                if(diffX > this.touchLimit) {
                    this.goPre();
                } else {
                    this._reset();
                }
            } else if(diffX < 0){
                // 左
                if(Math.abs(diffX) > this.touchLimit) {
                    this.goNext();
                } else {
                    this._reset();
                }
            } 
            isTouch = false;
        }

        SNKDom.bind({
            dom: this.itemsWrapper,
            type: eventDict.cancel,
            callback: (e) => {
                if(this.isInTransition) {
                    return;
                }
                touchEnd();
                diffX = 0;
            }
        });
    }

    _reset = ()=> {
        this.isInTransition = true;
        this.setPos(0,true);
        setTimeout(()=>{
            this.isInTransition = false;
        },300);
    }
    getPreSourceArr(){
        var len = this.data.length;
        var mid = this.sourceArr[1];
        mid -= 1;
        if(mid<0){
          if(this.isLoop){
            mid =len===1?0:len-1;
          }else{
            mid +=1;
          }
        }
        var lr = this.getLeftRightIndexByMid(mid,len);
        var arr = [lr.left,mid,lr.right,lr.rightright];
        this.sourceArr = arr;
      }
    
      getNextSourceArr(){
        var len = this.data.length;
        var mid = this.sourceArr[1];
        mid += 1;
        if(mid>len-1){
          if(this.isLoop){
            mid =len ===0?-1:0;
          }else{
            mid -=1;
          }
        }
        var lr = this.getLeftRightIndexByMid(mid,len);
        var arr = [lr.left,mid,lr.right,lr.rightright];
        this.sourceArr = arr;
      }
    
    
      getLeftRightIndexByMid(mid,len){
        var right;
        if(mid === -1){
          right = -1;
        }else{
          right = mid + 1;
          if(right>len-1){
            if(this.isLoop){
              right = len ===1?-1:0;
            }else{
              right = -1;
            }
          }
        }
    
        var rightright = -1;
        if(right===-1){
          rightright = -1;
        }else{
          rightright = right + 1;
          if(rightright>len-1){
            if(this.isLoop){
              rightright = len ===1?-1:0;
            }else{
              rightright = -1;
            }
          }
        }
    
        var left = mid - 1;
        if(left<0){
          if(this.isLoop){
            left =len ===1?-1:len-1;
          }else{
            left = -1;
          }
        }
        return {left:left,right:right,rightright}
    }
    goPre = () => {
        this.isInTransition = true;
        this.setPos(this.wrapperWidth,true);
        this._transitionEnd(true);
    }
    goNext = () => {
        this.isInTransition = true;
        this.setPos(0-this.wrapperWidth,true);
        this._transitionEnd(false);
    }
    _transitionEnd = (isPre) => {
        setTimeout(()=>{
            if(isPre){
                const re = this.itemArrary.pop();
                this.itemArrary.unshift(re);
                this.getPreSourceArr();
            }else {
                const re = this.itemArrary.shift();
                this.itemArrary.push(re);
                this.getNextSourceArr();
            }
            if(this.currentIndex!==this.sourceArr[1]){
                this.currentIndex = this.sourceArr[1];
                this.onChange({
                    isInit:false,
                    index:this.currentIndex,
                    slide:this,
                });
            }
            this.bindDataSource();
            this.offset = 0;
            this.setPos(0);
            this.isInTransition = false;
        },300);
    }
    bindDataSource = () => {
        for(let i = 0,j = this.itemArrary.length;i<j;i+=1) {
            const item = this.itemArrary[i];
            const sourceIndex = this.sourceArr[i];
            const source = this.data[sourceIndex];
            item.bind({
                data:source,
            });
        }
    }

    onChange = (params)=>{
      const { onChange } = this.props;
        if(this.thumbBarInstance) {
            this.thumbBarInstance.onChange(params);
        }
        const { index } = params;
        if(onChange) {
          onChange({index, data: this.data[index], slide: this});
        }
    }

    hide = () => {
        this.root.style.display = 'none';
        this.bk.style.display = 'none';
        this.isShow = false;
    }

    show = (params) => {
        this._show(params);
    }

    update = ({ data }) => {
      if(!this.isShow) {
        return;
      }
      const index = this.currentIndex -1 < 0?0:this.currentIndex -1;
      this._show({
        data,
        index,
      });
    }

    _show = (params) => {
        this.isShow = true;
        this.data = params.data || [];
        if(this.data.length === 0) {
            this.root.style.display = 'block';
            // alert('todo .. empty');
            return;
        }
        this.thumbBarInstance && this.thumbBarInstance.show({
            data:this.data
        });
        this.setSelectedByIndex(params.index);
        this.root.style.display = 'block';
        this.bk.style.display = 'block';
        this.wrapperWidth = this.root.offsetWidth;
        this.wrapperHeight = this.root.offsetHeight;
        this.setPos();
    }

    getThumbItemByIndex = (index) => {
      return this.thumbBarInstance.getThumbItemByIndex(index);
    }

    setPos = (offset,isAnimate) => {
        offset = offset || 0;
        this.offset = offset;
        for(let i = 0;i<3;i+=1) {
            const itemStyle = {};
            const item = this.itemArrary[i];
            if(isAnimate === true) {
                itemStyle[transformInfo.transition] = `left .3s ease`;
            } else {
                itemStyle[transformInfo.transition] = '';
            }
            itemStyle['left'] = `${(i-1)*this.wrapperWidth + offset}px`;
            item.setStyle(itemStyle);
        }
    }
}

export default Slide;