

import SNKDom from '../../dom';
import util from '../util';

export default class ImageItem {
    constructor(props){
        this.props = props;
        const { slide, data } = this.props;
        this.type = 'image';
        this.root = document.createElement("DIV");
        this.root.className = 'snk-slide-image-item';
        this.scale = 1;
        const { item } = data;
        const { snkrotateValue } = item.Vo;
        this.rotateValue = snkrotateValue || 0;
        let src = data.src;
        if(slide.props.getSrc) {
          src = slide.props.getSrc({
            data,
          });
        }
        this.loadImage(src);
    }

    rotate =(value) => {
      this.rotateValue = this.rotateValue + value;
      if(this.rotateValue >= 360) {
        this.rotateValue = 0;
      }
      if(this.rotateValue === -90) {
        this.rotateValue = 270;
      }
      this.reset();
    }
    bindEvent = () => {
        const {isSupportTouch,eventDict} = this.props;
        if(!isSupportTouch){
            SNKDom.bind({
                type: 'mousewheel',
                dom: this.root,
                callback: (e) => {
                    this.wheel(e);
                },
            });
            SNKDom.bind({
                type: 'DOMMouseScroll',
                dom: this.root,
                callback: (e) => {
                    this.wheel(e);
                },
            });
            SNKDom.bind({
              type: 'mousedown',
              dom: this.img,
              callback: (e) => {
                this.onMouseDown(e);
              },
            });
            SNKDom.bind({
              type: 'mousemove',
              dom: this.img,
              callback: (e) => {
                this.onMouseMove(e);
              },
            });
            SNKDom.bind({
              type: 'mouseup',
              dom: this.img,
              callback: (e) => {
                this.onMouseUp(e);
              },
            });
            
            SNKDom.bind({
              type: 'mouseleave',
              dom: this.img,
              callback: (e) => {
                this.onMouseUp(e);
              },
            });
        }
    }

    onMouseUp = (e) => {
      const { slide } = this.props;
      if(!slide.touchDisabled) {
        return;
      }
      e.preventDefault();
      e.stopPropagation();
      this.img.style.cursor = 'grab';
      this.isMouseDown = false;
    }

    onMouseMove = (e) => {
      const { slide } = this.props;
      if(!slide.touchDisabled || !this.isMouseDown) {
        return;
      }
      e.preventDefault();
      e.stopPropagation();
      var diff = {
        x:  e.pageX - this.start.x,
        y: e.pageY - this.start.y,
      };
      this.img.style.left = ( this.start.left + diff.x ) +'px';
      this.img.style.top = ( this.start.top + diff.y ) +'px';
    }

    onMouseDown = (e) => {
      const { slide } = this.props;
      if(!slide.touchDisabled) {
        return;
      }
      e.preventDefault();
      e.stopPropagation();
      this.isMouseDown = true;
      this.img.style.cursor = 'grabbing';
      this.start = {
        x: e.pageX,
        y: e.pageY,
        top:parseInt(this.img.style.top),
        left:parseInt(this.img.style.left),
      };
    }

    setSlideTouchDisabled = (isDisabled) => {
      const { slide } = this.props;
      slide.touchDisabled = isDisabled;
      if(isDisabled) {
        this.img.style.cursor = 'grab';
      } else {
        this.img.style.cursor = 'default';
      }
    }

    wheel = (e) => {
        e.preventDefault && e.preventDefault();
        // wheelDeltaY > 0 向前
        let detail = 0;
        if(e.wheelDelta){
            detail = e.wheelDelta;
        } else if (e.detail) {
            detail = 0-e.detail;
        }

        if(detail> 0 ){
            // 向上滚动
            this.scaleMethod(this.scale + 0.1,e);
        } else if(detail<0) {
            // 向下滚动
            this.scaleMethod(this.scale - 0.1,e);
        }
    }

    scaleMethod = (scale,e) => {
      const { slide } = this.props;
        const BeforeScalePointX = e.offsetX;
        const BeforeScalePointY = e.offsetY;
        scale = scale < 0 ? 0.1 : scale;
        
        const InOrignImagePointX = BeforeScalePointX/this.scale;
        const InOrignImagePointY = BeforeScalePointY/this.scale;
        this.scale = Math.round(scale*10) / 10;
        let beforeLeft = parseInt(this.img.style.left);
        let beforeTop = parseInt(this.img.style.top);
        const AfterScalePointX = InOrignImagePointX*this.scale;
        const AfterScalePointY = InOrignImagePointY*this.scale;
        const curWidth = this.imgOrginDomSize.width*this.scale;
        const curHeight = this.imgOrginDomSize.height*this.scale;
        this.img.style.width = `${curWidth}px`;
        this.img.style.height = `${curHeight}px`;
        this.img.style.left = (beforeLeft - (AfterScalePointX - BeforeScalePointX))+'px';
        this.img.style.top = (beforeTop - (AfterScalePointY - BeforeScalePointY))+'px';

        if(this.resetTimeout) {
            window.clearTimeout(this.resetTimeout);
            this.resetTimeout = null;
        }
        this.resetTimeout = setTimeout(()=>{
            if(this.scale <= 1) {
              this.setSlideTouchDisabled(false);
              this.reset();
            }else {
                this.setSlideTouchDisabled(true);
                const {  width:imgWidth, height:imgHeight } = this.img.getBoundingClientRect();
                const rootWidth = this.root.offsetWidth;
                const rootHeight = this.root.offsetHeight;
                if(imgWidth<rootWidth) {
                    this.img.style.left = `${(rootWidth-imgWidth)/2}px`;
                } else if(this.img.offsetLeft>0){
                    this.img.style.left = 0;
                } else if(imgWidth + this.img.offsetLeft< rootWidth) {
                    this.img.style.left = `${rootWidth - imgWidth}px`;
                }
                if(imgHeight<rootHeight) {
                    this.img.style.top = `${(rootHeight-imgHeight)/2}px`;
                } else if(this.img.offsetTop>0){
                    this.img.style.top = 0;
                } else if(imgHeight + this.img.offsetTop< rootHeight) {
                    this.img.style.top = `${rootHeight - imgHeight}px`;
                }
            }
        },150);
    }
    onResize = (size) => {
      const { slide } = this.props;
      this.setSlideTouchDisabled(false);
        this.scale = 1;
        this.autoResizeImage(size.width,size.height,this.imgOriginWidth,this.imgOriginHeight,this.img);
    }

    reset = () => {
      const { slide } = this.props;
      this.setSlideTouchDisabled(false);
        this.scale = 1;
        this.img.style.left = 0;
        this.img.style.top = 0;
        this.autoResizeImage(this.root.offsetWidth,this.root.offsetHeight,this.imgOriginWidth,this.imgOriginHeight,this.img);
    }

    autoResizeImage = (maxWidth,maxHeight,w,h,objImg) => {
      const { slide } = this.props;
      this.setSlideTouchDisabled(false);
        this.scale = 1;
        this.img.className = 'snk-slide-image';
        this.imgOrginDomSize = util.autoResizeImage(maxWidth,maxHeight,w,h,objImg,this.rotateValue, this.root);
    }
    loadImage = (src)=>{
        this.root.innerHTML = '';
        const loadingSpan = document.createElement("SPAN");
        loadingSpan.innerHTML = '加载中...';
        loadingSpan.className = 'snk-slide-image-loading';
        this.root.appendChild(loadingSpan);
        this.img = new Image();
        this.isUserWidth = false;
        this.img.className = 'snk-slide-image';
        this.img.onload = ()=>{
            this.scale = 1;
            setTimeout(()=>{
                this.imgOriginWidth = this.img.width;
                this.imgOriginHeight = this.img.height;
                this.wrapperWidth = this.root.offsetWidth;
                this.wrapperHeight = this.root.offsetHeight;
                this.root.innerHTML = '';
                this.root.appendChild(this.img);
                this.autoResizeImage(this.wrapperWidth,this.wrapperHeight,this.imgOriginWidth,this.imgOriginHeight,this.img)
                this.bindEvent(this.img);
            },10);
        }
        this.img.onerror = () => {
            this.root.innerHTML = '';
            const reload = document.createElement('SPAN');
            reload.className = 'snk-slide-image-loading';
            reload.innerHTML = '<span>网络异常,</span><span style="color:#2dd3d4;cursor:pointer;">点击重新加载</span>';
            this.root.appendChild(reload);
            SNKDom.bind({
                dom: reload,
                type: 'click',
                callback: () => {
                    this.loadImage(src);
                }
            });
        }
        this.img.src = src;
    }
}