import ImageItem from './imageItem';
import VideoItem from './videoItem';
import PDFItem from './pdfItem';
import NOTSupportItem from './notSupportItem';

export default class Item {
    constructor(props){
        this.props = props;
        this.root = document.createElement("DIV");
        this.root.className = 'snk-slide-item';
        props.wrapper.appendChild(this.root);
    }
    setStyle = (style) => {
        for(var key in style) {
            this.root.style[key] = style[key];
        }
    }
    onResize = (size)=> {
        if(this.itemInstance && this.itemInstance.onResize){
            this.itemInstance.onResize(size);
        }
    }
    bind = (params)=>{
        if(params.data && this.data!==params.data) {
            const {slide, isSupportTouch, eventDict} = this.props;
            this.data = params.data || {};
            let src = this.data.src || '';
            if(slide.props.getSrc){
              src = slide.props.getSrc({
                data: this.data,
              });
            }
            let subfix;
            let ItemClass = NOTSupportItem;
            if(slide.props.getFileType) {
              const type = slide.props.getFileType({
                src,
                data: this.data,
              });
              if(type) {
                subfix = type;
              }
            } 
            if(!subfix) {
              const srcArr = src.split(".");
              subfix = srcArr[srcArr.length - 1].toLowerCase();
              subfix = subfix.split('?')[0];
            }
            if(['jpg','png','gif','jpeg','bmp'].indexOf(subfix)>=0) {
                ItemClass = ImageItem;
            } else if (['pdf'].indexOf(subfix)>=0) {
                ItemClass = PDFItem;
            } else if (['mp4','rmvb'].indexOf(subfix)>=0) {
                ItemClass = VideoItem;
            }
            this.root.innerHTML = '';
            this.itemInstance = new ItemClass({
                src,
                data: this.data,
                slide:slide,
                isSupportTouch,
                parent: this,
                eventDict,
            });
            this.root.appendChild(this.itemInstance.root);
        }
    }
}