import SNKDom from '../../dom';
import TreeSection from './treeSection';

export default class TreeItem {
  constructor(props) {
    this.props = props;
    const {
      data, tree, popover, preDataArr, index,parentItem
    } = this.props;
    this.parentItem = parentItem;
    this.isSelected = false;
    this.id = tree.getUniqueSeed();
    tree.nodeDict[this.id] = this;
    this.isOpen = index === 0;
    this.root = document.createElement('LI');
    this.curPreArr = [...(preDataArr || [])];
    this.curPreArr.push(data);
    data.nodeInstance = this;
    this.createItemLayout(data);
    this.createChildSection(data);
    const params = {
      data,
      tree,
      node: this,
      preDataArr: this.curPreArr,
      popover,
    };
    SNKDom.bind({
      type: 'click',
      dom: this.root,
      prevent: true,
      stop: true,
      callback: () => {
        if (tree.props.itemClickToToggle === true) {
          this.toggle();
        }
        if (tree.props.onBeforeItemSelect) {
          if (tree.props.onBeforeItemSelect(params) !== false) {
            this.select();
          }
        } else {
          this.select();
        }
        if (tree.props.onItemClick) {
          tree.props.onItemClick(params);
        }
      },
    });
    if (tree.props.onItemCreate) {
      tree.props.onItemCreate(params);
    }
  }

  createChildSection = (data) => {
    const { children } = data;
    const { tree, index, popover } = this.props;
    if (!children) {
      return;
    }
    this.section = new TreeSection({
      tree,
      data: children,
      parentSection:this.parentSection,
      parentItem: this,
      index: index + 1,
      preDataArr: [...this.curPreArr],
      popover,
    });
    this.root.appendChild(this.section.root);
  }

  select = () => {
    const { tree } = this.props;
    if (tree.selectedNode) {
      tree.selectedNode.unSelect();
    }
    tree.selectedNode = this;
    this.isSelected = true;
    this.setClassName();
  }

  toSelect = () => {
    const {
      data, tree, popover
    } = this.props;
    const params = {
      data,
      tree,
      node: this,
      preDataArr: this.curPreArr,
      popover,
    };
    this.select();
    if (tree.props.onItemClick) {
      tree.props.onItemClick(params);
    }
  }

  unSelect = () => {
    this.isSelected = false;
    this.setClassName();
  }

  setClassName = () => {
    const {
      data,
      tree,
    } = this.props;
    const hasChildren = data.children && data.children.length > 0 && tree.props.setionItemNoHover;
    this.itemWrapper.className = `snk-tree-item${hasChildren ? '-header' : ''} ${this.isSelected ? 'snk-tree-item-selected' : ''} snk-tree-item-${this.isOpen ? 'open' : 'close'}`;
  }

  createItemLayout = (data) => {
    const { label, children } = data || { label: '' };
    const { index, tree } = this.props;
    const { isReact, onBeforeLabelCreate } = tree.props;
    this.itemWrapper = document.createElement('DIV');
    this.itemWrapper.setAttribute('node-id', this.id);
    this.setClassName();
    this.itemWrapper.style.paddingLeft = `${index * tree.offset}px`;
    if (children && children.length > 0) {
      const toggleIcon = document.createElement('SPAN');
      toggleIcon.className = 'snk-tree-toggle';
      this.itemWrapper.appendChild(toggleIcon);
      if (tree.props.itemClickToToggle !== true) {
        SNKDom.bind({
          dom: toggleIcon,
          type: 'click',
          stop: true,
          prevent: true,
          callback: () => {
            this.toggle();
          },
        });
      }
    } else {
      const toggleIconPlaceHolder = document.createElement('SPAN');
      toggleIconPlaceHolder.className = 'snk-tree-toggle-empty';
      this.itemWrapper.appendChild(toggleIconPlaceHolder);
    }
    const params = {
      tree,
      itemDom: this.itemWrapper,
      data,
      itemInstance: this,
    };
    if (isReact && onBeforeLabelCreate) {
      const reactBeforeLabelDom = document.createElement('span');
      reactBeforeLabelDom.className = 'snk-tree-span-beforelabel';
      this.itemWrapper.appendChild(reactBeforeLabelDom);
      params.reactBeforeLabelDom = reactBeforeLabelDom;
    }
    tree.beforeLabelCreate(params);

    this.countLabel = document.createElement('span');
    this.countLabel.className = 'snk-tree-span-countlabel';
    this.countLabel.innerHTML = 0;
    this.itemWrapper.appendChild(this.countLabel);

    const labelSpan = document.createElement('SPAN');

    
    labelSpan.className = 'snk-tree-label';
    labelSpan.innerHTML = label;
    labelSpan.setAttribute('title',label);
    this.itemWrapper.appendChild(labelSpan);

    this.root.appendChild(this.itemWrapper);
  }

  toggle = () => {
    if (this.isOpen) {
      this.close();
    } else {
      this.open();
    }
  }

  open = () => {
    if (!this.section) {
      return;
    }
    this.isOpen = true;
    this.setClassName();
    this.section.open();
  }

  close = () => {
    if (!this.section) {
      return;
    }
    this.isOpen = false;
    this.setClassName();
    this.section.close();
  }
}
