

import TreeItem from './treeItem';

class TreeSection {
  constructor(props) {
    this.props = props;
    const { data, index, parentItem } = this.props;
    this.isOpen = index === 1;
    this.root = document.createElement('DIV');
    this.root.className = 'snk-tree-section-wrapper';
    this.ul = document.createElement('UL');
    this.parentItem = parentItem;
    this.root.appendChild(this.ul);
    this.createLayout(data);
  }

  setClose = () => {
    this.isOpen = false;
    const { index } = this.props;
    if (index > 0) {
      this.root.style.height = '0px';
    }
  }

  toggle = () => {
    if (this.isOpen) {
      this.close();
    } else {
      this.open();
    }
  }

  open = () => {
    this.isOpen = true;
    this.root.style.height = '0px';
    setTimeout(() => {
      this.root.style.height = `${this.ul.offsetHeight}px`;
      setTimeout(() => {
        this.root.style.height = 'auto';
      }, 200);
    }, 0);
  }

  close = () => {
    this.isOpen = false;
    this.root.style.height = `${this.ul.offsetHeight}px`;
    setTimeout(() => {
      this.root.style.height = '0px';
    }, 0);
  }

  createLayout = (data) => {
    this.items = [];
    const {
      tree, index, popover, preDataArr,
    } = this.props;
    const frag = document.createDocumentFragment();
    for (let i = 0, j = data.length; i < j; i += 1) {
      const params = {
        data: data[i],
        tree,
        section: this,
        index,
        parentItem:this.parentItem,
        popover,
        preDataArr,
      };
      if (tree.props.onBeforeItemCreate) {
        if (tree.props.onBeforeItemCreate(params) === false) {
          // eslint-disable-next-line
          continue;
        }
      }
      const item = new TreeItem(params);
      item.parentSection = this;
      this.items.push(item);
      frag.appendChild(item.root);
    }
    this.ul.appendChild(frag);
    if (index !== 1) {
      this.setClose();
    }
  }
}

export default TreeSection;
