import SNKDom from '../dom';
import Tree from './tree';
import Popover from './components/popover';

class PopTree {
  constructor(props) {
    this.props = props || {};
    const { wrapper, className, popClassName } = this.props;
    if (!wrapper) {
      console.error('SNKTree 构造函数缺少wrapper字段');
    }
    this.offset = 10;
    this.root = document.createElement('DIV');
    const classNameArr = ['snk-poptree-root'];
    if (className) {
      classNameArr.push(className);
    }
    this.root.className = classNameArr.join(' ');
    wrapper.appendChild(this.root);
    const poplayerClassName = ['snk-tree-pop'];
    if (popClassName) {
      poplayerClassName.push(popClassName);
    }
    this.popInstance = new Popover({
      className: poplayerClassName.join(' '),
      onBKClick: () => {
        this.hide();
      },
      content: this.createTree(),
    });
    SNKDom.bind({
      type: 'click',
      dom: this.root,
      callback: () => {
        this.show();
      },
    });

    this.text = document.createElement('SPAN');
    this.text.className = 'snk-poptree-root-text';
    this.root.appendChild(this.text);
  }

  createTree = () => {
    const { onItemClick } = this.props;
    this.treeInstance = new Tree({
      ...this.props,
      popover: this,
      onItemClick: (params) => {
        this.setText(params.data.label || '');
        if (onItemClick) {
          onItemClick(params);
        }
      },
    });
    return this.treeInstance.root;
  }

  setText = (text) => {
    this.text.innerHTML = text;
    this.text.setAttribute('title', text);
  }

  hide = () => {
    this.popInstance.hide();
  }

  show = () => {
    this.popInstance.show({
      rect: this.root.getBoundingClientRect(),
    });
  }

  setSelected = (nodeInstance) => {
    if (this.selectedNode) {
      this.selectedNode.unSelect();
    }
    nodeInstance.select();
    this.setText(nodeInstance.props.data.label || '');
  }

  getSelectedNode = () => {
    return this.treeInstance.getSelectedNode();
  }

  loadData = () => {
    this.treeInstance.loadData({
      success: (res) => {
        if (res.data.length > 0) {
          this.setText(res.data[0].label);
        } else {
          this.setText('无相关数据');
        }
      },
    });
  }
}

export default PopTree;
