import { HttpClient } from 'lynnlin-libs';
import QueryString from 'querystring';

const httpClient = new HttpClient();

export default class HttpUtils {
  static _seq = 0;

  static _createUrl(params) {
    const server = params.server || SERVER;
    const url = typeof params === 'string' ? params : params.url;
    return server + url;
  }

  static async post(params) {
    // url, body
    const uri = HttpUtils._createUrl(params);
    const request = HttpUtils._createRequest(params.body);
    request.headers = params.headers || {};
    const response = await httpClient.post(uri, request);
    return HttpUtils._processResponse(response);
  }

  static async get(params) {
    let uri = HttpUtils._createUrl(params);
    if (params.body) uri += `?${QueryString.stringify(params.body)}`;
    const request = {
      headers: {
        Seq: (HttpUtils._seq += 1),
      },
    };
    request.headers = params.headers || {};
    const response = await httpClient.get(uri, request);
    return HttpUtils._processResponse(response);
  }

  static async put(params) {
    const uri = HttpUtils._createUrl(params);
    const request = HttpUtils._createRequest(params.body);
    const response = await httpClient.put(uri, request);
    return HttpUtils._processResponse(response);
  }

  static async delete(params) {
    const uri = HttpUtils._createUrl(params);
    const request = HttpUtils._createRequest(params.body);
    const response = await httpClient.delete(uri, request);
    return HttpUtils._processResponse(response);
  }

  static async upload(params) {
    // url body pcb
    const uri = HttpUtils._createUrl(params);
    const request = HttpUtils._createRequest(params.body);
    const response = await httpClient.upload(uri, request, params.pcb, { async: true });
    return HttpUtils._processResponse(response);
  }

  static formSubmit(params) {
    const uri = HttpUtils._createUrl(params);
    const { body } = params;
    const iframe = document.createElement('iframe');
    iframe.setAttribute('style', 'display:none');
    iframe.setAttribute('name', 'export_iframe');
    document.body.appendChild(iframe);
    const form = document.createElement('form');
    form.setAttribute('style', 'display:none');
    form.setAttribute('target', 'export_iframe');
    form.setAttribute('method', 'post');
    form.setAttribute('enctype', 'multipart/form-data');
    form.setAttribute('action', uri);
    if (body) {
      Object.keys(body).forEach((key) => {
        const element = body[key];
        const input = document.createElement('input');
        input.setAttribute('style', 'display:none');
        input.setAttribute('type', 'hidden');
        input.setAttribute('name', key);
        input.setAttribute('value', element);
        form.appendChild(input);
      });
    }

    document.body.appendChild(form);
    form.submit();
  }

  static _createRequest(body) {
    const request = {
      headers: {
        Seq: (HttpUtils._seq += 1),
      },
      body,
    };
    return request;
  }

  static async _processResponse(response) {
    const json = await response.json();
    const { code, message, data } = json;
    if (code !== 0) {
      const err = { code, message };
      throw err;
    } else {
      return data;
    }
  }
}
